#include <iostream>
#include <fstream>
#include <cmath>
#include <algorithm>
#include <unistd.h>
#include "odeSolvers.hpp"

double fTest(double t, double y) {
  return y*y*t;
}

double exactSol(double t, double y0) {
  return 2*y0/(2-t*t*y0);
}

void printUsageAndQuit(char* exeName) {
  std::cout << "Usage: "                                              << std::endl
            << "  " << exeName << " -o <outputFile> <dt1> <dt2> ..."  << std::endl
            << "where dt1, dt2, are small real numbers (time steps)." << std::endl;
  exit(EXIT_SUCCESS);
}

void readCommandLine(int argc, char** argv, std::string& fileName, std::vector<double>& timeSteps) {

  int opt;

  // default file name for results
  fileName = "res.dat";

  // Parse options
  while ((opt = getopt(argc, argv, "ho:")) != -1) {
    switch (opt) {
      case 'o':
        fileName = std::string(optarg);
        break;
      case 'h':
      default:
        printUsageAndQuit(argv[0]);
    }
  }

  // Check that there are some time steps
  if (optind >= argc)
    printUsageAndQuit(argv[0]);

  // Read time steps
  for (int i=optind; i<argc; i++)
    timeSteps.push_back(std::stod(argv[i]));

}

int main(int argc, char* argv[]) {

  int N = 1000;
  // Data vector at different step
  rvec y0(N),y1(N),y2(N);

  // Vectors of time steps and associated errors
  std::vector<double> dts;
  std::vector<double> errs;

  std::string(fileName);

  // Get time steps and output file name
  readCommandLine(argc,argv,fileName,dts);

  // A file in which to put the results
  std::ofstream oFile(fileName);

  for (double dt: dts) {

    // "Simulation" times
    double t      = 0.;
    double tFinal = 1.;

    // Initial condition
    for (int i=0; i<N; i++)
      y0[i] = -sin(M_PI/2. *i/(double)(N-1));
 
    // Compute exact answer for later
    rvec sol(N);
    for (int i=0; i<N; i++)
      sol[i] = exactSol(tFinal,y0[i]);
 
    // First step is performed with Euler scheme
    oneStepEuler(y1,y0,&fTest,dt,t);
    t += dt;
 
    // Pointers on data vector for faster swapping
    std::vector<rvec*> yN(3);
    yN[2] = &y2;
    yN[1] = &y1;
    yN[0] = &y0;
 
    // Now we use Adams-Bashforth method for all remaining steps
    while (t<tFinal) {
      // Compute next step
      oneStepAdamsBashforth2(*(yN[2]),*(yN[1]),*(yN[0]),&fTest,dt,t);
      // Shift pointers
      std::rotate(yN.begin(),yN.begin()+1,yN.end());
      t += dt;
    }
 
    // Compute the approximation error at the end
    double normDiff = 0.;
    double normSol  = 0.;
    for (int i=0; i<sol.size(); i++) {
      normDiff += pow((*yN[2])[i]-sol[i],2);
      normSol  += pow(sol[i],2);
    }

    errs.push_back(sqrt(normDiff/normSol));
    std::cout << "dt: \t" << dt << " \t error: " << errs.back() << std::endl;
    oFile << dt << " \t" << errs.back() << std::endl;

  }

  return 0;

}

