#ifndef _SPARSE_MATRIX_HPP_
#define _SPARSE_MATRIX_HPP_

#include <cmath>
#include <vector>

typedef unsigned int uint;

struct sparseMatrix {

  uint size;  // Size of whole matrix
  uint nNz;   // # of non zero terms

  std::vector<uint  > rows,cols; // row and column indices of non zero terms
  std::vector<double> vals;      // non zero terms

};

// Fills the (NxN) sparse matrix with the discretization of a 1D heat equation
// dt u - laplace u = 0
// with Dirichlet BC for first and last coefficients (suits with our case)
void fill1DHeat(sparseMatrix& M, double dx, double dt, int N); 

// Reads coefficients of a sparse matrix from a text file
// The file contains the number of non zero terms on its first line
// then on each line the row index, column index and value of non zero term
sparseMatrix readMatrix(std::string fileName) ;

// Prints on screen the contents of a sparse matrix
void displayMatrix(const sparseMatrix& M) ;

// Computes trace of matrix (i.e. sum of diagonal terms)
double trace(const sparseMatrix& M);

// Computes infinite norm of matrix (max over rows of sum of abs of coefficients)
double inftNorm(const sparseMatrix& M);

// Fills res with the matrix vector product Mx.
void matMul(const sparseMatrix& M, const std::vector<double>& x, std::vector<double>& res);

double inftNorm(const std::vector<double>& x);

// Uses power iteration method to compute dominating eigen value of sparse matrix
double powerIteration(const sparseMatrix& M, int maxIters = 100, double tolerance = 1.e-4);

// Simple scalar product
double dotProd(const std::vector<double>& v1, const std::vector<double>& v2);

// Solves the linear system Ax=b with conjugate gradient where
// matrix is sparse and vectors dense
// Convergence is tested with absolute and relative tolerance
void solveCG(const sparseMatrix& A, const std::vector<double>& b, std::vector<double>& x);

#endif//_SPARSE_MATRIX_HPP_
